@@ -76,13 +76,33 @@ class MutableNotebookSafetySettings:
 
 
 class CheckerResult(NamedTuple):
-    live: Set[DataSymbol]
-    deep_live: Set[DataSymbol]
-    shallow_live: Set[DataSymbol]
-    live_cells: Set[int]
-    live_cells_from_calls: Set[int]
-    dead: Set[DataSymbol]
-    stale: Set[DataSymbol]
+    live: Set[DataSymbol]           # all live symbols in the cell
+    deep_live: Set[DataSymbol]      # live symbols used in their entirety
+    shallow_live: Set[DataSymbol]   # live symbols for which only a portion (attr or subscript) is used
+    used_cells: Set[int]            # last updated timestamps of the live symbols
+    live_cells: Set[int]            # cells that define a symbol that was called in the cell
+    dead: Set[DataSymbol]           # symbols that are definitely assigned to
+    stale: Set[DataSymbol]          # live symbols that have one or more ancestors with more recent timestamps
+
+
+class FrontendCheckerResult(NamedTuple):
+    stale_cells: Set[CellId]
+    fresh_cells: Set[CellId]
+    new_fresh_cells: Set[CellId]
+    stale_links: Dict[CellId, Set[CellId]]
+    refresher_links: Dict[CellId, Set[CellId]]
+    phantom_cell_info: Dict[CellId, Dict[CellId, Set[int]]]
+
+    def to_json(self) -> Dict[str, Any]:
+        return {
+            'stale_cells': list(self.stale_cells),
+            'fresh_cells': list(self.fresh_cells),
+            'new_fresh_cells': list(self.new_fresh_cells),
+            'stale_links': {cell_id: list(linked_cell_ids) for cell_id, linked_cell_ids in self.stale_links.items()},
+            'refresher_links': {
+                cell_id: list(linked_cell_ids) for cell_id, linked_cell_ids in self.refresher_links.items()
+            },
+        }
 
 
 class NotebookSafety(singletons.NotebookSafety):
@@ -153,9 +173,13 @@ class NotebookSafety(singletons.NotebookSafety):
             get_ipython().kernel.comm_manager.register_target(__package__, self._comm_target)
 
     @property
-    def current_cell_content(self):
+    def current_cell_content(self) -> str:
         return self.cell_content_by_counter[self.cell_counter()]
 
+    @property
+    def active_cell_id(self) -> CellId:
+        return self._cell_id_by_counter[self.cell_counter()]
+
     @property
     def is_develop(self) -> bool:
         return self.settings.mode == SafetyRunMode.DEVELOP
@@ -246,7 +270,7 @@ class NotebookSafety(singletons.NotebookSafety):
             else:
                 order_index_by_id = request['order_index_by_cell_id']
                 last_cell_exec_position_idx = order_index_by_id.get(cell_id, -1)
-            response = self.check_and_link_multiple_cells(cells_by_id, order_index_by_id)
+            response = self.check_and_link_multiple_cells(cells_by_id, order_index_by_id).to_json()
             response['type'] = 'cell_freshness'
             response['last_cell_exec_position_idx'] = last_cell_exec_position_idx
             response['exec_mode'] = self.mut_settings.exec_mode.value
@@ -259,29 +283,41 @@ class NotebookSafety(singletons.NotebookSafety):
             logger.error(dbg_msg)
             self._saved_debug_message = dbg_msg
 
+    def _compute_phantom_cell_info(self, cell_id: CellId, used_cells: Set[int]) -> Dict[CellId, Set[int]]:
+        used_cell_counters_by_cell_id = defaultdict(set)
+        used_cell_counters_by_cell_id[cell_id].add(self.cell_counter())
+        for cell_num in used_cells:
+            used_cell_counters_by_cell_id[self._cell_id_by_counter[cell_num]].add(cell_num)
+        return {
+            cell_id: cell_execs
+            for cell_id, cell_execs in used_cell_counters_by_cell_id.items()
+            if len(cell_execs) >= 2
+        }
+
     def check_and_link_multiple_cells(
-        self,
-        content_by_cell_id: Optional[Dict[CellId, str]] = None,
-        order_index_by_cell_id: Optional[Dict[CellId, int]] = None
-    ) -> Dict[str, Any]:
+            self,
+            content_by_cell_id: Optional[Dict[CellId, str]] = None,
+        order_index_by_cell_id: Optional[Dict[CellId, int]] = None,
+        update_liveness_time_versions: bool = False,
+    ) -> FrontendCheckerResult:
         if content_by_cell_id is None:
             content_by_cell_id = self.cell_content_by_cell_id
         stale_cells = set()
-        fresh_cells = []
-        new_fresh_cells = []
+        fresh_cells = set()
+        new_fresh_cells = set()
         stale_symbols_by_cell_id: Dict[CellId, Set[DataSymbol]] = {}
         killing_cell_ids_for_symbol: Dict[DataSymbol, Set[CellId]] = defaultdict(set)
         cell_ids_needing_typecheck = self.get_cell_ids_needing_typecheck()
+        phantom_cell_info: Dict[CellId, Dict[CellId, Set[int]]] = {}
         for cell_id, cell_content in content_by_cell_id.items():
             if cell_id not in self._run_cells:
                 continue
-            if (
-                order_index_by_cell_id is not None
-                and order_index_by_cell_id.get(cell_id, -1) <= self.active_cell_position_idx
-            ):
-                continue
             try:
-                checker_result = self._check_cell_and_resolve_symbols(cell_content, self._counter_by_cell_id[cell_id])
+                checker_result = self._check_cell_and_resolve_symbols(
+                    cell_content,
+                    self._counter_by_cell_id[cell_id],
+                    update_liveness_time_versions=update_liveness_time_versions,
+                )
                 stale_symbols, dead_symbols = checker_result.stale, checker_result.dead
                 if len(stale_symbols) > 0:
                     stale_symbols_by_cell_id[cell_id] = stale_symbols
@@ -299,26 +335,27 @@ class NotebookSafety(singletons.NotebookSafety):
                             self._typecheck_error_cells.discard(cell_id)
                         else:
                             self._typecheck_error_cells.add(cell_id)
-                    except Exception as e:
-                        logger.info('Exception ocurred during type checking: %s', e)
+                    except Exception:
+                        logger.exception('Exception occurred during type checking')
                         self._typecheck_error_cells.discard(cell_id)
 
                 if self.settings.mark_phantom_cell_usages_unsafe:
-                    # TODO: implement phantom cell usage detection here
-                    pass
+                    phantom_cell_info_for_cell = self._compute_phantom_cell_info(cell_id, checker_result.used_cells)
+                    if len(phantom_cell_info_for_cell) > 0:
+                        phantom_cell_info[cell_id] = phantom_cell_info_for_cell
 
                 if cell_id not in stale_cells and cell_id not in self._typecheck_error_cells:
                     max_timestamp_cell_num = self._get_max_timestamp_cell_num_for_symbols(
                         checker_result.deep_live, checker_result.shallow_live
                     )
                     if max_timestamp_cell_num > self._counter_by_cell_id.get(cell_id, cast(int, float('inf'))):
-                        fresh_cells.append(cell_id)
+                        fresh_cells.add(cell_id)
                     if max_timestamp_cell_num >= self._last_execution_counter:
-                        new_fresh_cells.append(cell_id)
+                        new_fresh_cells.add(cell_id)
             except SyntaxError:
                 continue
         stale_links: Dict[CellId, Set[CellId]] = defaultdict(set)
-        refresher_links: Dict[CellId, List[CellId]] = defaultdict(list)
+        refresher_links: Dict[CellId, Set[CellId]] = defaultdict(set)
         for stale_cell_id in stale_cells:
             stale_syms = stale_symbols_by_cell_id[stale_cell_id]
             refresher_cell_ids: Set[CellId] = set()
@@ -343,7 +380,7 @@ class NotebookSafety(singletons.NotebookSafety):
         for stale_cell_id in stale_cells:
             stale_links[stale_cell_id] -= stale_cells
             for refresher_cell_id in stale_links[stale_cell_id]:
-                refresher_links[refresher_cell_id].append(stale_cell_id)
+                refresher_links[refresher_cell_id].add(stale_cell_id)
         if self.settings.mark_typecheck_failures_unsafe:
             # TODO: actually do the type checking
             #  nested symbols in particular will need some thought
@@ -351,18 +388,16 @@ class NotebookSafety(singletons.NotebookSafety):
         for typecheck_error_cell in self._typecheck_error_cells:
             if typecheck_error_cell not in stale_links:
                 stale_links[typecheck_error_cell] = set()
-        return {
+        return FrontendCheckerResult(
             # TODO: we should probably have separate fields for stale vs non-typechecking cells,
             #  or at least change the name to a more general "unsafe_cells" or equivalent
-            'stale_cells': list(stale_cells | self._typecheck_error_cells),
-            'fresh_cells': fresh_cells,
-            'new_fresh_cells': new_fresh_cells,
-            'stale_links': {
-                stale_cell_id: list(refresher_cell_ids)
-                for stale_cell_id, refresher_cell_ids in stale_links.items()
-            },
-            'refresher_links': refresher_links,
-        }
+            stale_cells=stale_cells | self._typecheck_error_cells,
+            fresh_cells=fresh_cells,
+            new_fresh_cells=new_fresh_cells,
+            stale_links=stale_links,
+            refresher_links=refresher_links,
+            phantom_cell_info=phantom_cell_info,
+        )
 
     @staticmethod
     def _get_cell_ast(cell):
@@ -392,7 +427,7 @@ class NotebookSafety(singletons.NotebookSafety):
         self, cell_id: CellId, content_by_cell_id: Dict[CellId, str], checker_result: CheckerResult
     ) -> str:
         live_cell_counters = {self._counter_by_cell_id[cell_id]}
-        for live_cell_num in checker_result.live_cells_from_calls:
+        for live_cell_num in checker_result.live_cells:
             live_cell_id = self._cell_id_by_counter[live_cell_num]
             if self._counter_by_cell_id[live_cell_id] == live_cell_num:
                 live_cell_counters.add(live_cell_num)
@@ -416,7 +451,7 @@ class NotebookSafety(singletons.NotebookSafety):
             get_live_symbols_and_cells_for_references(
                 live_symbol_refs, self.global_scope, cell_ctr, update_liveness_time_versions=True
             )
-        deep_live_symbols, shallow_live_symbols, live_cells_from_calls = get_live_symbols_and_cells_for_references(
+        deep_live_symbols, shallow_live_symbols, live_cells = get_live_symbols_and_cells_for_references(
             live_symbol_refs, self.global_scope, cell_ctr, update_liveness_time_versions=False
         )
         live_symbols = deep_live_symbols | shallow_live_symbols
@@ -426,12 +461,19 @@ class NotebookSafety(singletons.NotebookSafety):
         )
         stale_symbols = {dsym for dsym in live_symbols if dsym.is_stale}
         self.live_symbols_by_cell_counter[cell_ctr] = live_symbols
+        if update_liveness_time_versions:
+            for sym in live_symbols:
+                self.cell_counter_by_live_symbol[sym].add(cell_ctr)
         return CheckerResult(
             live=live_symbols,
             deep_live=deep_live_symbols,
             shallow_live=shallow_live_symbols,
-            live_cells={sym.timestamp.cell_num for sym in live_symbols},
-            live_cells_from_calls=live_cells_from_calls,
+            used_cells={
+                sym.timestamp.cell_num for sym in deep_live_symbols
+            } | {
+                sym.timestamp_excluding_ns_descendents.cell_num for sym in shallow_live_symbols
+            },
+            live_cells=live_cells,
             dead=dead_symbols,
             stale=stale_symbols,
         )
@@ -458,14 +500,21 @@ class NotebookSafety(singletons.NotebookSafety):
         )
 
     @staticmethod
-    def _stale_cell_warning(cell_execs: Set[int]):
+    def _phantom_cell_warning(cell_execs: Set[int]):
         if len(cell_execs) < 2:
             raise ValueError('Expected cell that was executed at least twice')
         logger.warning(
             f'Detected usages of symbols across same cell executed multiple times (timestamps {cell_execs}) '
         )
 
-    def _safety_precheck_cell(self, cell: str, cell_id: Optional[CellId]) -> bool:
+    def _safety_precheck_cell(self, cell: str, cell_id: Optional[CellId]) -> None:
+        checker_result = self.check_and_link_multiple_cells(
+            {cell_id: cell}, update_liveness_time_versions=self.mut_settings.static_slicing_enabled
+        )
+        if cell_id in checker_result.stale_cells:
+            self.safety_issue_detected = True
+
+    def _safety_precheck_cell_old(self, cell: str, cell_id: Optional[CellId]) -> bool:
         """
         This method statically checks the cell to be executed for stale symbols and other safety issues.
         If any safety issues are detected, it returns `True`. Otherwise (or on syntax error), return `False`.
@@ -483,7 +532,7 @@ class NotebookSafety(singletons.NotebookSafety):
         checker_result = self._check_cell_and_resolve_symbols(
             cell_ast, self.cell_counter(), update_liveness_time_versions=self.mut_settings.static_slicing_enabled
         )
-        stale_symbols, live_symbols, live_cells = checker_result.stale, checker_result.live, checker_result.live_cells
+        stale_symbols, live_symbols, used_cells = checker_result.stale, checker_result.live, checker_result.used_cells
         stale_sym_usage_warning_counter = 0
         phantom_cell_usage_warning_counter = 0
         if self._last_refused_code is None or cell != self._last_refused_code:
@@ -499,21 +548,13 @@ class NotebookSafety(singletons.NotebookSafety):
                 if stale_sym_usage_warning_counter > 0:
                     logger.warning('\n\n(Run cell again to override and execute anyway.)')
             if self.settings.mark_phantom_cell_usages_unsafe:
-                used_cell_counters_by_cell_id = defaultdict(set)
-                used_cell_counters_by_cell_id[cell_id].add(self.cell_counter())
-                for cell_num in live_cells:
-                    used_cell_counters_by_cell_id[self._cell_id_by_counter[cell_num]].add(cell_num)
-                phantom_cell_info = {
-                    cell_id: cell_execs
-                    for cell_id, cell_execs in used_cell_counters_by_cell_id.items()
-                    if len(cell_execs) >= 2
-                }
+                phantom_cell_info = self._compute_phantom_cell_info(cell_id, used_cells)
                 for used_cell_execs in phantom_cell_info.values():
                     if phantom_cell_usage_warning_counter >= self._MAX_STALE_CELL_WARNINGS:
                         logger.warning(f'{len(phantom_cell_info) - phantom_cell_usage_warning_counter} '
                                        'more cells with symbol usages from phantom cells skipped...')
                         break
-                    self._stale_cell_warning(used_cell_execs)
+                    self._phantom_cell_warning(used_cell_execs)
                     phantom_cell_usage_warning_counter += 1
             if self.settings.mark_typecheck_failures_unsafe:
                 # TODO: implement typechecking here, same as in `check_and_link_multiple_cells`
@@ -778,26 +819,27 @@ class NotebookSafety(singletons.NotebookSafety):
             self._last_execution_counter = self.cell_counter()
 
             cell_id, self._active_cell_id = self._active_cell_id, None
-            if cell_id is not None:
-                self._counter_by_cell_id[cell_id] = self._last_execution_counter
-                self._cell_id_by_counter[self._last_execution_counter] = cell_id
-                # add to run_cells even if it fails precheck, since an attempt
-                # to run indicates that the user is interested in this cell, so
-                # we should add this cell to the set of cells that get checked
-                # in various ways in the post-exec hook.
-                self._run_cells.add(cell_id)
+            assert cell_id is not None
+            self._counter_by_cell_id[cell_id] = self._last_execution_counter
+            self._cell_id_by_counter[self._last_execution_counter] = cell_id
+            # add to run_cells even if it fails precheck, since an attempt
+            # to run indicates that the user is interested in this cell, so
+            # we should add this cell to the set of cells that get checked
+            # in various ways in the post-exec hook.
+            self._run_cells.add(cell_id)
 
             # Stage 1: Precheck.
-            if self._safety_precheck_cell(cell, cell_id) and self.settings.mark_stale_symbol_usages_unsafe:
-                # set this back in case we need it (e.g. user overrides and reruns)
-                self._active_cell_id = cell_id
-                # FIXME: hack to increase cell number
-                #  ideally we shouldn't show a cell number at all if we fail precheck since nothing executed
-                if is_async:
-                    ret = await run_cell_func('None')  # pragma: no cover
-                else:
-                    ret = run_cell_func('None')
-                return ret
+            self._safety_precheck_cell(cell, cell_id)
+            # if self._safety_precheck_cell_old(cell, cell_id):
+            #     # set this back in case we need it (e.g. user overrides and reruns)
+            #     self._active_cell_id = cell_id
+            #     # FIXME: hack to increase cell number
+            #     #  ideally we shouldn't show a cell number at all if we fail precheck since nothing executed
+            #     if is_async:
+            #         ret = await run_cell_func('None')  # pragma: no cover
+            #     else:
+            #         ret = run_cell_func('None')
+            #     return ret
 
             # Stage 2: Trace / run the cell, updating dependencies as they are encountered.
             try:
