@@ -31,6 +31,10 @@ from nbsafety.tracing.mutation_event import (
     StandardMutation,
     resolve_mutating_method,
 )
+from nbsafety.tracing.mutation_special_cases import (
+    METHODS_WITHOUT_MUTATION_EVEN_FOR_NULL_RETURN,
+    METHODS_WITH_MUTATION_EVEN_FOR_NON_NULL_RETURN,
+)
 from nbsafety.tracing.symbol_resolver import resolve_rval_symbols
 from nbsafety.tracing.trace_events import TraceEvent
 from nbsafety.tracing.trace_stack import TraceStack
@@ -71,26 +75,6 @@ ARG_MUTATION_EXCEPTED_MODULES = {
 }
 
 
-METHODS_WITH_MUTATION_EVEN_FOR_NON_NULL_RETURN: Set[Tuple[int, str]] = set()
-METHODS_WITHOUT_MUTATION_EVEN_FOR_NULL_RETURN: Set[Tuple[int, str]] = set()
-
-try:
-    import pylab
-    pylab_id = id(pylab)
-    METHODS_WITH_MUTATION_EVEN_FOR_NON_NULL_RETURN.add((pylab_id, 'figure'))
-    METHODS_WITHOUT_MUTATION_EVEN_FOR_NULL_RETURN.add((pylab_id, 'show'))
-except ImportError:
-    pass
-
-try:
-    import matplotlib.pyplot as plt
-    plt_id = id(plt)
-    METHODS_WITH_MUTATION_EVEN_FOR_NON_NULL_RETURN.add((plt_id, 'figure'))
-    METHODS_WITHOUT_MUTATION_EVEN_FOR_NULL_RETURN.add((plt_id, 'show'))
-except ImportError:
-    pass
-
-
 class BaseTraceManager(singletons.TraceManager):
 
     _MANAGER_CLASS_REGISTERED = False
@@ -283,7 +267,7 @@ class TraceManager(SliceTraceManager):
             self.prev_trace_stmt_in_cur_frame: Optional[TraceStatement] = None
             self.prev_node_id_in_cur_frame: Optional[NodeId] = None
             self.mutations: List[Mutation] = []
-            self.saved_assign_rhs_obj_id: Optional[int] = None
+            self.saved_assign_rhs_obj: Optional[Any] = None
             # this one gets set regardless of whether tracing enabled
             self.next_stmt_node_id: Optional[NodeId] = None
 
@@ -323,6 +307,7 @@ class TraceManager(SliceTraceManager):
         self.node_id_to_loaded_literal_scope.clear()
         self.node_id_to_saved_dict_key.clear()
         self.prev_node_id_in_cur_frame = None
+        self.saved_assign_rhs_obj = None
         # don't clear the lexical stacks because line magics can
         # mess with when an 'after_stmt' gets emitted, and anyway
         # these should be pushed / popped appropriately by ast events
@@ -446,42 +431,51 @@ class TraceManager(SliceTraceManager):
             ref = id(ref)
         return ref
 
+    def _resolve_store_data_for_simple_target(self, target: str, frame: FrameType):
+        scope = self.cur_frame_original_scope
+        lut = frame.f_locals
+        if scope.symtab is not None:
+            try:
+                target_sym = scope.symtab.lookup(target)
+                # this nonsense is necessary because the "is_nonlocal" method
+                # is not available on Python <= 3.7;
+                # the below check seems to work consistently across all Python versions
+                is_nonlocal = getattr(
+                    target_sym, 'is_nonlocal',
+                    lambda: not target_sym.is_global()
+                            and target_sym.is_assigned()
+                            and target_sym.is_free()
+                )()
+                if is_nonlocal:
+                    scope = scope.parent_scope
+                elif target_sym.is_global():
+                    lut = frame.f_globals
+                    scope = nbs().global_scope
+            except KeyError:
+                pass
+        try:
+            obj = lut[target]
+        except KeyError:
+            obj = frame.f_globals[target]
+            scope = nbs().global_scope
+        return scope, target, obj, False, set()
+
     def resolve_store_data_for_target(
         self, target: Union[str, int, ast.AST], frame: FrameType
     ) -> Tuple[Scope, AttrSubVal, Any, bool, Set[DataSymbol]]:
         target = self._partial_resolve_ref(target)
         if isinstance(target, str):
-            scope = self.cur_frame_original_scope
-            lut = frame.f_locals
-            if scope.symtab is not None:
-                try:
-                    target_sym = scope.symtab.lookup(target)
-                    # this nonsense is necessary because the "is_nonlocal" method
-                    # is not available on Python <= 3.7;
-                    # the below check seems to work consistently across all Python versions
-                    is_nonlocal = getattr(
-                        target_sym, 'is_nonlocal',
-                        lambda: not target_sym.is_global()
-                                    and target_sym.is_assigned()
-                                    and target_sym.is_free()
-                    )()
-                    if is_nonlocal:
-                        scope = scope.parent_scope
-                    elif target_sym.is_global():
-                        lut = frame.f_globals
-                        scope = nbs().global_scope
-                except KeyError:
-                    pass
-            try:
-                obj = lut[target]
-            except KeyError:
-                obj = frame.f_globals[target]
-                scope = nbs().global_scope
-            return scope, target, obj, False, set()
+            return self._resolve_store_data_for_simple_target(target, frame)
         (
             scope, obj, attr_or_sub, is_subscript
         ) = self.node_id_to_saved_store_data.pop(target)
-        attr_or_sub_obj = nbs().retrieve_namespace_attr_or_sub(obj, attr_or_sub, is_subscript)
+        if isinstance(obj, (dict, list)):
+            # we can be reasonably sure that the object on the rhs is the same thing
+            # that gets stashed in `obj` for these cases, so use it instead of doing
+            # the lookup (which may have side effects) to reduce intrusiveness
+            attr_or_sub_obj = self.saved_assign_rhs_obj
+        else:
+            attr_or_sub_obj = nbs().retrieve_namespace_attr_or_sub(obj, attr_or_sub, is_subscript)
         if attr_or_sub_obj is None:
             scope_to_use = scope
         else:
@@ -625,7 +619,7 @@ class TraceManager(SliceTraceManager):
     @register_handler(TraceEvent.after_assign_rhs)
     @skip_when_tracing_disabled
     def after_assign_rhs(self, obj: Any, *_, **__):
-        self.saved_assign_rhs_obj_id = id(obj)
+        self.saved_assign_rhs_obj = obj
 
     @register_handler(TraceEvent.subscript_slice)
     @skip_when_tracing_disabled
