@@ -30,12 +30,13 @@ def override_settings(**kwargs):
         nbs().settings = old_settings
 
 
-def run_cell(cell, cell_id=None, **kwargs):
-    """Mocks the `change active cell` portion of the comm protocol"""
-    if cell_id is None:
-        cell_id = nbs().cell_counter()
-    nbs().set_active_cell(cell_id)
-    run_cell_(cell, **kwargs)
+def run_cell(cell, cell_id, **kwargs):
+    # print()
+    # print('*******************************************')
+    # print('running', cell)
+    # print('*******************************************')
+    # print()
+    run_cell_(cell, cell_id=cell_id, **kwargs)
 
 
 def run_all_cells(cells: Dict[int, str], **kwargs):
@@ -52,10 +53,10 @@ def test_simple():
     }
     run_all_cells(cells)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == [2]
-    assert response['fresh_cells'] == [1]
-    assert response['stale_links'] == {2: [1]}
-    assert response['refresher_links'] == {1: [2]}
+    assert response.stale_cells == {2}
+    assert response.fresh_cells == {1}
+    assert response.stale_links == {2: {1}}
+    assert response.refresher_links == {1: {2}}
 
 
 def test_refresh_after_exception_fixed():
@@ -68,7 +69,7 @@ def test_refresh_after_exception_fixed():
     run_cell(cells[2], 2, ignore_exceptions=True)
     run_cell(cells[1], 1)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['fresh_cells'] == [2]
+    assert response.fresh_cells == {2}
 
 
 def test_refresh_after_val_changed():
@@ -80,7 +81,7 @@ def test_refresh_after_val_changed():
     }
     run_all_cells(cells)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['fresh_cells'] == [2]
+    assert response.fresh_cells == {2}
 
 
 def test_inner_mutation_considered_fresh():
@@ -93,8 +94,8 @@ def test_inner_mutation_considered_fresh():
     }
     run_all_cells(cells)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == [2, 3]
+    assert response.stale_cells == set()
+    assert response.fresh_cells == {2, 3}
 
 
 # @pytest.mark.parametrize("force_subscript_symbol_creation", [True, False])
@@ -126,33 +127,33 @@ for foo in lst:
     run_all_cells(cells)
 
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == [], 'got %s' % response['fresh_cells']
+    assert response.stale_cells == set()
+    assert response.fresh_cells == set(), 'got %s' % response.fresh_cells
 
     cells[4] = 'x.inc()'
     run_cell(cells[4], 4)
 
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == [], 'got %s' % response['stale_cells']
-    assert response['fresh_cells'] == [2, 3], 'got %s' % response['fresh_cells']
+    assert response.stale_cells == set(), 'got %s' % response.stale_cells
+    assert response.fresh_cells == {2, 3}, 'got %s' % response.fresh_cells
 
     cells[5] = 'foo.inc()'
     run_cell(cells[5], 5)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == [], 'got %s' % response['stale_cells']
-    assert response['fresh_cells'] == [2, 3, 4], 'got %s' % response['fresh_cells']
+    assert response.stale_cells == set(), 'got %s' % response.stale_cells
+    assert response.fresh_cells == {2, 3, 4}, 'got %s' % response.fresh_cells
 
     if force_subscript_symbol_creation:
         cells[6] = 'lst[-1]'
         run_cell(cells[6], 6)
         response = nbs().check_and_link_multiple_cells(cells)
-        assert response['stale_cells'] == []
-        assert response['fresh_cells'] == [2, 3, 4]
+        assert response.stale_cells == set()
+        assert response.fresh_cells == {2, 3, 4}
 
     run_cell(cells[4], 4)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == [2, 3, 5] + ([6] if force_subscript_symbol_creation else [])
+    assert response.stale_cells == set()
+    assert response.fresh_cells == set([2, 3, 5] + ([6] if force_subscript_symbol_creation else []))
 
 
 def test_no_freshness_for_alias_assignment_post_mutation():
@@ -163,8 +164,8 @@ def test_no_freshness_for_alias_assignment_post_mutation():
     }
     run_all_cells(cells)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == []
+    assert response.stale_cells == set()
+    assert response.fresh_cells == set()
 
 
 def test_fresh_after_import():
@@ -174,8 +175,8 @@ def test_fresh_after_import():
     }
     run_all_cells(cells, ignore_exceptions=True)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == [0]
+    assert response.stale_cells == set()
+    assert response.fresh_cells == {0}
 
 
 def test_external_object_update_propagates_to_stale_namespace_symbols():
@@ -191,8 +192,8 @@ def test_external_object_update_propagates_to_stale_namespace_symbols():
     with override_settings(mark_stale_symbol_usages_unsafe=False):
         run_all_cells(cells)
         response = nbs().check_and_link_multiple_cells(cells)
-        assert response['stale_cells'] == [], 'got %s' % response['stale_cells']
-        assert response['fresh_cells'] == [2, 4]
+        assert response.stale_cells == set(), 'got %s' % response.stale_cells
+        assert response.fresh_cells == {2, 4}
 
 
 def test_symbol_on_both_sides_of_assignment():
@@ -204,9 +205,9 @@ def test_symbol_on_both_sides_of_assignment():
     }
     run_all_cells(cells)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == [2]
-    assert response['fresh_cells'] == [1]
-    assert list(response['refresher_links'].keys()) == [1]
+    assert response.stale_cells == {2}
+    assert response.fresh_cells == {1}
+    assert list(response.refresher_links.keys()) == [1]
 
 
 def test_updated_namespace_after_subscript_dep_removed():
@@ -218,21 +219,21 @@ def test_updated_namespace_after_subscript_dep_removed():
     }
     run_all_cells(cells)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == [2]
-    assert response['fresh_cells'] == [1]
+    assert response.stale_cells == {2}
+    assert response.fresh_cells == {1}
     cells[1] = 'd = {5: 6}'
     run_cell(cells[1], 1)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == [2]
+    assert response.stale_cells == set()
+    assert response.fresh_cells == {2}
     run_cell(cells[2], 2)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == []
+    assert response.stale_cells == set()
+    assert response.fresh_cells == set()
     run_cell(cells[0], 0)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == [], 'got %s' % response['fresh_cells']
+    assert response.stale_cells == set()
+    assert response.fresh_cells == set(), 'got %s' % response.fresh_cells
 
 
 def test_equal_list_update_does_induce_fresh_cell():
@@ -244,12 +245,12 @@ def test_equal_list_update_does_induce_fresh_cell():
     }
     run_all_cells(cells)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == [2]
+    assert response.stale_cells == set()
+    assert response.fresh_cells == {2}
     run_cell('y = ("f",)', 4)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == [2, 3]
+    assert response.stale_cells == set()
+    assert response.fresh_cells == {2, 3}
 
 
 def test_equal_list_update_does_induce_fresh_cell_LITERAL_WITH_F_IS_REUSED_ON_UBUNTU_20_04_2_PYTHON_3_8_11():
@@ -261,12 +262,12 @@ def test_equal_list_update_does_induce_fresh_cell_LITERAL_WITH_F_IS_REUSED_ON_UB
     }
     run_all_cells(cells)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == [2]
+    assert response.stale_cells == set()
+    assert response.fresh_cells == {2}
     run_cell('y = ["f"]', 4)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == [2, 3]
+    assert response.stale_cells == set()
+    assert response.fresh_cells == {2, 3}
 
 
 def test_equal_dict_update_does_induce_fresh_cell():
@@ -278,12 +279,12 @@ def test_equal_dict_update_does_induce_fresh_cell():
     }
     run_all_cells(cells)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == [2], 'got %s' % response['fresh_cells']
+    assert response.stale_cells == set()
+    assert response.fresh_cells == {2}, 'got %s' % response.fresh_cells
     run_cell('y = {"foo": 99}', 4)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == [2, 3]
+    assert response.stale_cells == set()
+    assert response.fresh_cells == {2, 3}
 
 
 def test_list_append():
@@ -295,12 +296,12 @@ def test_list_append():
     }
     run_all_cells(cells)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == []
+    assert response.stale_cells == set()
+    assert response.fresh_cells == set()
     run_cell('lst[1] += 42', 4)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == [2]
-    assert response['fresh_cells'] == [1]
+    assert response.stale_cells == {2}
+    assert response.fresh_cells == {1}
 
 
 def test_list_extend():
@@ -312,12 +313,12 @@ def test_list_extend():
     }
     run_all_cells(cells)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == []
+    assert response.stale_cells == set()
+    assert response.fresh_cells == set()
     run_cell('lst[1] += 42', 4)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == [2]
-    assert response['fresh_cells'] == [1]
+    assert response.stale_cells == {2}
+    assert response.fresh_cells == {1}
 
 
 def test_implicit_subscript_symbol_does_not_bump_ts():
@@ -328,8 +329,8 @@ def test_implicit_subscript_symbol_does_not_bump_ts():
     }
     run_all_cells(cells)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == []
+    assert response.stale_cells == set()
+    assert response.fresh_cells == set()
 
 
 def test_liveness_skipped_for_simple_assignment_involving_aliases():
@@ -340,12 +341,12 @@ def test_liveness_skipped_for_simple_assignment_involving_aliases():
     }
     run_all_cells(cells)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == []
+    assert response.stale_cells == set()
+    assert response.fresh_cells == set()
     run_cell('lst = [1, 2, 3, 4]', 3)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == [1, 2], 'got %s' % response['fresh_cells']
+    assert response.stale_cells == set()
+    assert response.fresh_cells == {1, 2}, 'got %s' % response.fresh_cells
 
 
 def test_incorrect_object_not_used_for_argument_symbols():
@@ -358,8 +359,8 @@ def test_incorrect_object_not_used_for_argument_symbols():
     }
     run_all_cells(cells)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == [], 'got %s' % response['fresh_cells']
+    assert response.stale_cells == set()
+    assert response.fresh_cells == set(), 'got %s' % response.fresh_cells
 
 
 def test_increment_by_same_amount():
@@ -370,12 +371,12 @@ def test_increment_by_same_amount():
     }
     run_all_cells(cells)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == []
+    assert response.stale_cells == set()
+    assert response.fresh_cells == set()
     run_cell('x = 3', 0)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == [2]
-    assert response['fresh_cells'] == [1]
+    assert response.stale_cells == {2}
+    assert response.fresh_cells == {1}
 
 
 def test_list_insert():
@@ -393,8 +394,8 @@ def test_list_insert():
     }
     run_all_cells(cells)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == [8], 'got %s' % response['stale_cells']
-    assert response['fresh_cells'] == [4, 5, 6, 7], 'got %s' % response['fresh_cells']
+    assert response.stale_cells == {8}, 'got %s' % response.stale_cells
+    assert response.fresh_cells == {4, 5, 6, 7}, 'got %s' % response.fresh_cells
 
 
 def _test_list_delete_helper(last_cell):
@@ -414,12 +415,12 @@ def _test_list_delete_helper(last_cell):
     }
     run_all_cells(cells)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == [10]
+    assert response.stale_cells == {10}
     # TODO: ideally we would detect that lst[3] is the same after deleting
     #  and not consider cell 4 to be fresh
     # TODO: ideally cell 8 would be considered unsafe since the last entry
     #  no longer exists
-    assert response['fresh_cells'] == [4, 5, 6, 7, 9], 'got %s' % response['fresh_cells']
+    assert response.fresh_cells == {4, 5, 6, 7, 9}, 'got %s' % response.fresh_cells
 
 
 def test_list_delete():
@@ -442,12 +443,12 @@ def test_list_clear():
     }
     run_all_cells(cells)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == []
+    assert response.stale_cells == set()
+    assert response.fresh_cells == set()
     run_cell('lst.clear()', 3)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == [2]
+    assert response.stale_cells == set()
+    assert response.fresh_cells == {2}
 
 
 def test_dict_clear():
@@ -458,12 +459,12 @@ def test_dict_clear():
     }
     run_all_cells(cells)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == []
+    assert response.stale_cells == set()
+    assert response.fresh_cells == set()
     run_cell('d.clear()', 3)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == [2]
+    assert response.stale_cells == set()
+    assert response.fresh_cells == {2}
 
 
 def test_adhoc_pandas_series_update():
@@ -475,9 +476,9 @@ def test_adhoc_pandas_series_update():
     }
     run_all_cells(cells)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == []
-    run_cell('df["foo"] = [8, 9]')
+    assert response.stale_cells == set()
+    assert response.fresh_cells == set()
+    run_cell('df["foo"] = [8, 9]', 4)
     response = nbs().check_and_link_multiple_cells(cells)
-    assert response['stale_cells'] == []
-    assert response['fresh_cells'] == [3]
+    assert response.stale_cells == set()
+    assert response.fresh_cells == {3}
